"""django_simepar_translation_server URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.10/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""
from django.conf.urls import url, include
from translation_server.views import *
from rest_framework import routers

router = routers.DefaultRouter()
router.register(r'translation', TranslationViewSet, 'translations')
router.register(r'translation_type', TranslationTypeViewSet, 'translations_types')

urlpatterns = [
    url(r'^api/last_translation_tag/(?P<tag>\w+)[/]?$', LastTranslationTagView.as_view(),
        name='get_last_translation_tag'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2017-01-12 18:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('translation_server', '0003_auto_20161021_1303'),
    ]

    operations = [
        migrations.RenameField(
            model_name='translation',
            old_name='created_at',
            new_name='created',
        ),
        migrations.RenameField(
            model_name='translation',
            old_name='updated_at',
            new_name='updated',
        ),
        migrations.RenameField(
            model_name='translationtype',
            old_name='created_at',
            new_name='created',
        ),
        migrations.RenameField(
            model_name='translationtype',
            old_name='updated_at',
            new_name='updated',
        ),
    ]



# -*- coding: utf-8 -*-
# Created by Gustavo Del Negro <gustavodelnegro@gmail.com> on 10/1/16.
import django
from django.core.management import BaseCommand
from django.apps import *
from datetime import datetime
from django.core.management import call_command
from django.conf import settings
import os
import glob
from translation_server.models import Translation, TranslationType


class Command(BaseCommand):
    help = "This command generates migrations for translations, based on the contents of 'Translation' model"
    app_name = None
    BASE_DIR = settings.BASE_DIR
    updated_translations = []
    updated_translation_types = []

    migration_string = """# -*- coding: utf-8 -*-
# Generated by Django %(django_version)s on %(timestamp)s
from __future__ import unicode_literals

from django.db import migrations


def __create_translation_types(**kwargs):
    apps = kwargs.pop('apps', None)
    if apps:
        model = apps.get_model("%(app_name)s", "TranslationType")
        try:
            translation_type = model.objects.get(tag=kwargs['tag'])
        except model.DoesNotExist:
            translation_type = model()
        except Exception as err:
            raise
        for k,v in kwargs.items():
            setattr(translation_type, k, v)
        translation_type.save()


def __load_data(**kwargs):
    apps = kwargs.pop('apps', None)
    if apps:
        translation_type = apps.get_model("%(app_name)s", "TranslationType")
        model = apps.get_model("%(app_name)s", "Translation")
        try:
            mdl = model.objects.get(tag=kwargs['tag'])
        except model.DoesNotExist:
            mdl = model()
        except Exception as err:
            raise err
        for k, v in kwargs.items():
            if k == "type":
                setattr(mdl, k, translation_type.objects.get(tag=v))
            else:
                setattr(mdl, k, v)
        setattr(mdl, "migration_created", True)
        mdl.save()


def clear_data(apps, schema_editor):
    model = apps.get_model("%(app_name)s", "Translation")
    model.objects.filter(tag__in=[%(tags_to_remove)s]).delete()
    translation_type = apps.get_model("%(app_name)s", "TranslationType")
    translation_type.objects.filter(tag__in=[%(translation_types_to_remove)s]).delete()


def load_data(apps, schema_editor):
%(translation_type_strings)s
%(translation_strings)s


class Migration(migrations.Migration):

    dependencies = [
        %(dependency_string)s
    ]

    operations = [
        migrations.RunPython(load_data, clear_data)
    ]

        """

    def __create_translation_type_lines(self):
        new_lines = []
        fields_to_ignore = ['id', 'created', 'updated']
        fields_string = ""
        fields_options = {}
        model_fields = [field.name if field.name not in fields_to_ignore else None for field in
                      apps.get_model('translation_server', "TranslationType")._meta.get_fields()]
        for field in model_fields:
            if field:
                fields_string += ', %s="%%(%s)s"' % (field, field)
                fields_options.update({field: ''})
        base_str = '    __create_translation_types(apps=apps%(fields)s)' % {'fields': fields_string}

        for translation in TranslationType.objects.all():
            for field in fields_options:
                value = getattr(translation, field)
                if type(value) is str:
                    value = value.replace('"', '\\"') if len(value) > 0 else ""
                if field == 'type':
                    value = value.tag
                if field == 'migration_created':
                    value = True
                fields_options[field] = value
            new_lines.append(base_str % fields_options)
            self.updated_translation_types.append(translation.tag)
        return new_lines

    def __create_translation_lines(self):
        new_lines = []
        fields_to_ignore = ['id', 'created', 'updated']
        fields_string = ""
        fields_options = {}
        model_fields = [field.name if field.name not in fields_to_ignore else None for field in
                      apps.get_model('translation_server', "Translation")._meta.get_fields()]
        for field in model_fields:
            if field:
                fields_string += ', %s="%%(%s)s"' % (field, field)
                fields_options.update({field: ''})
        base_str = '    __load_data(apps=apps%(fields)s)' % {'fields': fields_string}

        for translation in Translation.objects.filter(migration_created=False):
            for field in fields_options:
                value = getattr(translation, field)
                if type(value) is str:
                    value = value.replace('"', '\\"') if len(value) > 0 else ""
                if field == 'type':
                    value = value.tag
                if field == 'migration_created':
                    value = True
                fields_options[field] = value
            new_lines.append(base_str % fields_options)
            self.updated_translations.append(translation.tag)
        return new_lines

    def __update_translation(self):
        for tag in self.updated_translations:
            translation = Translation.objects.get(tag=tag)
            translation.migration_created = True
            translation.save()

    def __create_translation_migration(self):
        """ Create an empty migration """
        migrations_dir = os.path.join(self.BASE_DIR, self.app_name + "/migrations/")
        dependency_migration = os.path.basename(max(glob.iglob(migrations_dir + '*.py'), key=os.path.getctime)).replace(
            ".py", "")
        """
        If there's no migration before this, which is unlikely to happen, then create a migration without dependencies
        """
        if "__init__" in dependency_migration:
            dependency_migration = ""
        """ Make an empty migration """
        call_command('makemigrations', self.app_name, "--empty")
        """ Get last migration name and edit it, adding the new code """
        last_migration_file = max(glob.iglob(migrations_dir + '*.py'), key=os.path.getctime)
        new_lines = self.__create_translation_lines()
        translation_type_lines = self.__create_translation_type_lines()
        if len(dependency_migration) > 0:
            dependency_string = "('%(app_name)s', '%(dependency)s')," % {'app_name': self.app_name,
                                                                         'dependency': dependency_migration}
        else:
            dependency_string = ""
        try:
            if len(new_lines) > 0:
                with open(last_migration_file, 'w+') as file:
                    file.write(self.migration_string % {
                        'django_version': django.get_version(),
                        'timestamp': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
                        'translation_strings': "\n".join(new_lines),
                        'tags_to_remove': ", ".join('"{0}"'.format(tag) for tag in self.updated_translations),
                        'dependency_string': dependency_string,
                        'app_name': 'translation_server',
                        'translation_type_strings': "\n".join(translation_type_lines),
                        'translation_types_to_remove': ", ".join(
                            '"{0}"'.format(tag) for tag in self.updated_translation_types),
                    })
            else:
                os.remove(last_migration_file)
                self.stdout.write(self.style.NOTICE("There was no new translations to make migrations"))
                return
        except Exception as error:
            os.remove(last_migration_file)
            raise error
        else:
            self.__update_translation()
            self.stdout.write(self.style.SUCCESS("Translation migration file create successfully"))
            return

    def add_arguments(self, parser):
        parser.add_argument('app_name', type=str)

    def handle(self, *args, **options):
        self.app_name = options['app_name']
        self.__create_translation_migration()



# -*- coding: utf-8 -*-
# Created by Gustavo Del Negro <gustavodelnegro@gmail.com> on 10/1/16.
from django.core.management import BaseCommand
from django.apps import *
from django.core.management import call_command
from django.conf import settings
import re
import os


class Command(BaseCommand):
    help = "This command import a csv to the database"

    app_name = "translation_server"

    csv_file = None

    def __create_record(self, model_name, *args, **kwargs):
        model = apps.get_model('translation_server', model_name)
        try:
            mdl = model.objects.get(tag=kwargs['tag'])
        except model.DoesNotExist:
            mdl = model()
        except Exception as err:
            raise err
        for k, v in kwargs.items():
            if k == "type":
                setattr(mdl, k, apps.get_model("translation_server", "TranslationType").objects.get(tag=v))
            else:
                setattr(mdl, k, v)
        try:
            mdl.save()
        except Exception as err:
            raise err

    def __import_csv(self):
        """
        file format:
        header
        Código WMLB|Código WMHT|WMLB EN|WMLB PT|WMHT EN|WMHT PT|Nome do Campo|Tipo do Campo|Contexto|Local de Uso
        primary_tag|auxiliary_tag|text_en|text_pt_br|auxiliary_text_en|auxiliary_text_pt_br|Nome do Campo|Tipo do Campo|Contexto|Local de Uso
        0|1|2|3|4|5|6|7|8|9
        """
        # valids = re.sub(r"[^A-Za-z]+", '', my_string)
        # if "código" in line, then it's header, ignore it
        # type = re.sub(r"[^A-Za-z]+", '', line[0])
        # auxiliary_type = re.sub(r"[^A-Za-z]+", '', line[1])
        import sys
        import csv

        with open(self.csv_file, 'r') as f:
            has_auxiliary_text = False
            reader = csv.reader(f)
            for row in reader:
                # Translation Type
                if len(row) == 2:
                    translation_type_tag = None
                    auxiliary_translation_type_tag = None
                    has_auxiliary_text = False
                    types_list = row[0].split("e")
                    translation_type_tag = types_list[0].strip()
                    if len(types_list) == 2:
                        auxiliary_translation_type_tag = types_list[1].strip()
                        has_auxiliary_text = True
                    self.__create_record("TranslationType",
                                         **{'tag': translation_type_tag,
                                            'name': translation_type_tag,
                                            'has_auxiliary_text': has_auxiliary_text,
                                            'auxiliary_tag': auxiliary_translation_type_tag,
                                            'name_en': translation_type_tag,
                                            'name_pt_br': translation_type_tag})
                else:
                    if "Código" not in row[0] and len(row) > 1:
                        if len(row) == 5:
                            params_dict = {
                                'type': re.sub(r"[^A-Za-z]+", '', row[0]),
                                'tag': row[0],
                                'text': row[1],
                                'text_en': row[1],
                                'text_pt_br': row[2],
                            }
                        else:
                            params_dict = {
                                'type': re.sub(r"[^A-Za-z]+", '', row[0]),
                                'tag': row[0],
                                'text': row[2],
                                'text_en': row[2],
                                'text_pt_br': row[3],
                            }
                            if has_auxiliary_text:
                                params_dict.update({
                                    'auxiliary_tag': row[1],
                                    'auxiliary_text': row[4],
                                    'auxiliary_text_en': row[4],
                                    'auxiliary_text_pt_br': row[5]
                                })
                        self.__create_record("Translation", **params_dict)

    def add_arguments(self, parser):
        parser.add_argument(
            'file',
            action='store',
            default="",
            help="The input csv file",
        )

    def handle(self, *args, **options):
        if len(options['file']) > 0:
            self.csv_file = options['file']
            try:
                my_file = open(self.csv_file)
            except IOError as error:
                raise error
            else:
                self.__import_csv()



# -*- coding: utf-8 -*-
# Created by Gustavo Del Negro <gustavodelnegro@gmail.com> on 10/1/16.
from django.core.management import BaseCommand
from django.core.management import call_command
from django.conf import settings


class Command(BaseCommand):
    help = "This command executes all steps for translation"

    app_name = "translation_server"

    def add_arguments(self, parser):
        parser.add_argument(
            '--locales-dir',
            action='store',
            dest='locales_dir',
            default="",
            help="The locales dir to copy the translation files",
        )
        parser.add_argument(
            '--migrate',
            action='store',
            dest='migrate',
            default=False,
            help="If the translation migrations should be made",
        )

    def handle(self, *args, **options):
        languages_list = [lang[0] for lang in settings.LANGUAGES]
        if options['migrate']:
            call_command('make_translation_migrations', self.app_name)
            call_command('migrate', self.app_name)
        call_command('make_translation', ",".join(languages_list), locales_dir=options['locales_dir'])


# -*- coding: utf-8 -*-
# Created by Gustavo Del Negro <gustavodelnegro@gmail.com> on 10/2/16.
from django.core.management import call_command
from django.test import TestCase
from django.conf import settings
from django.core.management.base import CommandError
from django.utils.six import StringIO
import os
import sys


class TestManagementCommands(TestCase):

    languages_list = [lang[0] for lang in settings.LANGUAGES]
    app_name = 'translation_server'
    LOCALE_PATH = os.path.dirname(os.path.abspath(__file__)) + "/../locale"

    def test_make_translation(self):
        """
        Test command make_translation
        """
        call_command('make_translation', ",".join(self.languages_list))

    def test_make_translation_migrations(self):
        """
        Test command make_translation
        """
        call_command('make_translation_migrations', self.app_name)

    def test_translate_without_args(self):
        """
        Test command translate without args
        """
        args = []
        opts = {}
        call_command('translate', *args, **opts)

    def test_translate_with_args(self):
        """
        Test command make_translation with args
        """
        args = []
        opts = {}
        call_command('translate', locales_dir=self.LOCALE_PATH)

    def test_upload_csv_with_invalid_file(self):
        """
        Test command upload_csv with invalid file
        """
        out = StringIO()
        sys.sdout = out
        file = os.path.dirname(os.path.abspath(__file__)) + "/../../../assets/TableExpor.csv"
        self.assertRaises(IOError, call_command, 'upload_csv', file)

    def test_upload_csv_with_valid_file(self):
        """
        Test command upload_csv with valid file
        """
        out = StringIO()
        sys.sdout = out
        file = os.path.dirname(os.path.abspath(__file__)) + "/../../../assets/TableExport.csv"
        call_command('upload_csv', file, stdout=out)
        self.assertEqual("", out.getvalue())


# -*- coding: utf-8 -*-
# Created by Gustavo Del Negro <gustavodelnegro@gmail.com> on 10/1/16.
from modeltranslation.translator import TranslationOptions, translator
from translation_server.models import *


class TranslationModelTranslationOptions(TranslationOptions):
    fields = ('text', 'auxiliary_text')
    required_languages = {'en': ('text', ), 'pt-br': ('text',)}


class TranslationTypeModelTranslationOptions(TranslationOptions):
    fields = ('name',)
    required_languages = ('en', 'pt-br')


class SiteSettingsModelTranslationOptions(TranslationOptions):
    fields = ('value',)
    required_languages = ('en', 'pt-br')

translator.register(Translation, TranslationModelTranslationOptions)
translator.register(TranslationType, TranslationTypeModelTranslationOptions)


from django.contrib import admin
from modeltranslation.admin import TabbedTranslationAdmin
from django.conf import settings
from translation_server.models import *
from translation_server.forms import *


class CustomModelAdminMixin(object):
    def __init__(self, model, admin_site):

        self.list_display = [field.name for field in model._meta.fields if field.name != "id"]
        super(CustomModelAdminMixin, self).__init__(model, admin_site)
        self.filter_horizontal = [field.name for field in model._meta.get_fields() if
                                  field.many_to_many and "reverse" not in str(type(field))]

        # get all fields that aren't related to another model
        self.search_fields = [field.name for field in model._meta.get_fields() if
                              not field.many_to_many and "Point" not in str(type(field)) and "related" not in str(
                                  type(field))]

        # get all languages installed on settings.py
        languages_list = [lang[0].replace("-", "_") for lang in settings.LANGUAGES]

        related_fields = []
        # get all fields that are related to another model
        for model_field in model._meta.get_fields():
            if not model_field.many_to_many and "related" in str(type(model_field)):
                for related_field in model_field.related_model._meta.get_fields():
                    # if the field have one of the fields from the 'related_fields_to_add' variable
                    # and one of the languages in the field name, add it to the search_fields
                    if any("_"+language_code in related_field.name for language_code in languages_list):
                        related_fields.append(model_field.name+"__"+related_field.name)
                        self.search_fields.append(model_field.name+"__"+related_field.name)

        self.search_fields.remove("id")


@admin.register(TranslationType)
class TranslationTypeAdmin(CustomModelAdminMixin, TabbedTranslationAdmin):
    def get_queryset(self, request):
        qs = super(TranslationTypeAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.exclude(tag__startswith='DTS')

    class Media:
        import os
        js_dir = os.path.join(settings.STATIC_URL, 'admin/js')
        js = (
            js_dir + '/admin-translation-type.js',
        )


@admin.register(Translation)
class TranslationAdmin(CustomModelAdminMixin, TabbedTranslationAdmin):
    form = TranslationAdminForm
    fieldsets = (
        ('Translation type', {
            'fields': ('type', 'translations_url', 'translation_type_url', 'last_translation_tag_url')
        }),
        ('Primary info', {
            'fields': ('tag', 'text')
        }),
        ('Auxiliary info', {
            'fields': ('auxiliary_tag', 'auxiliary_text')
        }),
    )
    list_display = ('tag', 'type', 'text')

    class Media:
        import os
        js_dir = os.path.join(settings.STATIC_URL, 'admin/js')
        js = (
            js_dir + '/admin-translation.js',
        )

    def get_queryset(self, request):
        qs = super(TranslationAdmin, self).get_queryset(request)
        # if request.user.is_superuser:
        #     return qs
        return qs.exclude(tag__startswith='DTS')




# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-21 13:03:04
from __future__ import unicode_literals

from django.db import migrations


def __load_data(**kwargs):
    apps = kwargs.pop('apps', None)
    if apps:
        translation_type = apps.get_model("translation_server", "TranslationType")
        model = apps.get_model("translation_server", "Translation")
        try:
            mdl = model.objects.get(tag=kwargs['tag'])
        except model.DoesNotExist:
            mdl = model()
        except Exception as err:
            raise err
        for k, v in kwargs.items():
            if k == "type":
                setattr(mdl, k, translation_type.objects.get(tag=v))
            else:
                setattr(mdl, k, v)
        setattr(mdl, "migration_created", True)
        mdl.save()


def clear_data(apps, schema_editor):
    model = apps.get_model("translation_server", "Translation")
    model.objects.filter(tag__in=["DTSM5", "DTSMT2", "DTSE1", "DTSM12", "DTSMT1", "DTSM1", "DTSM8", "DTSM9", "DTSM7", "DTSM10", "DTSM11", "DTSM2", "DTSM3", "DTSM4", "DTSM6"]).delete()


def load_data(apps, schema_editor):
    __load_data(apps=apps, type="DTSM", tag="DTSM5", text="Has auxiliary text?", text_en="Has auxiliary text?", text_pt_br="Tem texto auxiliar?", auxiliary_tag="DTST5", auxiliary_text="If the translation type have auxiliary text", auxiliary_text_en="If the translation type have auxiliary text", auxiliary_text_pt_br="Se o tipo de tradução tem texto auxiliar", migration_created="True")
    __load_data(apps=apps, type="DTSMT", tag="DTSMT2", text="Translation", text_en="Translation", text_pt_br=u"Tradução", auxiliary_tag="DTSMTP2", auxiliary_text="Translations", auxiliary_text_en="Translations", auxiliary_text_pt_br=u"Traduções", migration_created="True")
    __load_data(apps=apps, type="DTSE", tag="DTSE1", text="The auxiliary text must be different from the primary text", text_en="The auxiliary text must be different from the primary text", text_pt_br="O texto auxiliar deve ser diferente do texto principal", auxiliary_tag="", auxiliary_text="", auxiliary_text_en="", auxiliary_text_pt_br="", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM12", text="Migration created", text_en="Migration created", text_pt_br="\"Migration\" criada", auxiliary_tag="DTST12", auxiliary_text="If this record migration was created", auxiliary_text_en="If this record migration was created", auxiliary_text_pt_br="Se a migration desse registro foi criada", migration_created="True")
    __load_data(apps=apps, type="DTSMT", tag="DTSMT1", text="Translation Type", text_en="Translation Type", text_pt_br="Tipo de tradução", auxiliary_tag="DTSMTP1", auxiliary_text="Translation Types", auxiliary_text_en="Translation Types", auxiliary_text_pt_br="Tipos de tradução", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM1", text="Created at", text_en="Created at", text_pt_br="Criado em", auxiliary_tag="DTST1", auxiliary_text="Record creation date", auxiliary_text_en="Record creation date", auxiliary_text_pt_br="Data de criação do registro", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM8", text="Tag", text_en="Tag", text_pt_br="Tag", auxiliary_tag="DTST8", auxiliary_text="Unique identifier", auxiliary_text_en="Unique identifier", auxiliary_text_pt_br="Identificador único", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM9", text="Text", text_en="Text", text_pt_br="Texto", auxiliary_tag="DTST9", auxiliary_text="Translation text", auxiliary_text_en="Translation text", auxiliary_text_pt_br="Texto da tradução", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM7", text="Translation Type", text_en="Translation Type", text_pt_br="Tipo de tradução", auxiliary_tag="DTST7", auxiliary_text="Record translation type", auxiliary_text_en="Record translation type", auxiliary_text_pt_br="Tipo de tradução do registro", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM10", text="Auxiliary tag", text_en="Auxiliary tag", text_pt_br="Tag auxiliar", auxiliary_tag="DTST10", auxiliary_text="Auxiliary text unique identifier", auxiliary_text_en="Auxiliary text unique identifier", auxiliary_text_pt_br="Identificador único do texto auxiliar", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM11", text="Auxiliary text", text_en="Auxiliary text", text_pt_br="Texto auxiliar", auxiliary_tag="DTST11", auxiliary_text="Auxiliary text", auxiliary_text_en="Auxiliary text", auxiliary_text_pt_br="Texto auxiliar", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM2", text="Updated at", text_en="Updated at", text_pt_br="Atualizado em", auxiliary_tag="DTST2", auxiliary_text="Record update time", auxiliary_text_en="Record update time", auxiliary_text_pt_br="Data de atualização do registro", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM3", text="Tag", text_en="Tag", text_pt_br="Tag", auxiliary_tag="DTST3", auxiliary_text="Unique identifier that will be used as prefix for the translations tag", auxiliary_text_en="Unique identifier that will be used as prefix for the translations tag", auxiliary_text_pt_br="Identificador único que será usado como prefixo da tag das traduções", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM4", text="Name", text_en="Name", text_pt_br="Nome", auxiliary_tag="DTST4", auxiliary_text="Record name", auxiliary_text_en="Record name", auxiliary_text_pt_br="Nome do registro", migration_created="True")
    __load_data(apps=apps, type="DTSM", tag="DTSM6", text="Auxiliary text tag", text_en="Auxiliary text tag", text_pt_br="Tag do texto auxiliar", auxiliary_tag="DTST6", auxiliary_text="The unique identifier prefix for auxiliary text", auxiliary_text_en="The unique identifier prefix for auxiliary text", auxiliary_text_pt_br="O prefixo para o  identificador único do texto auxiliar", migration_created="True")


class Migration(migrations.Migration):

    dependencies = [
        ('translation_server', '0002_auto_20161021_1254'),
    ]

    operations = [
        migrations.RunPython(load_data, clear_data)
    ]

        



# -*- coding: utf-8 -*-
# Created by Gustavo Del Negro <gustavodelnegro@gmail.com> on 10/1/16.
from translation_server.models import *
from rest_framework import serializers


class TranslationTypeSerializer(serializers.ModelSerializer):
    class Meta:
        model = TranslationType
        fields = ('tag', 'has_auxiliary_text', 'auxiliary_tag',)


class TranslationSerializer(serializers.ModelSerializer):
    type = TranslationTypeSerializer()

    class Meta:
        model = Translation
        fields = "__all__"



# -*- coding: utf-8 -*-
# Created by Gustavo Del Negro <gustavodelnegro@gmail.com> on 9/30/16.
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from translation_server.models import *
from django.urls import reverse


class TranslationAdminForm(forms.ModelForm):
    languages_list = [lang[0].replace('-', '_') for lang in settings.LANGUAGES]
    translations_url = forms.CharField(max_length=200, widget=forms.HiddenInput())
    translation_type_url = forms.CharField(max_length=200, widget=forms.HiddenInput())
    last_translation_tag_url = forms.CharField(max_length=200, widget=forms.HiddenInput())

    def __init__(self, *args, **kwargs):
        instance = kwargs.get('instance', None)
        kwargs.update(initial={
            'translations_url': reverse('api:translations-list'),
            'translation_type_url': reverse('api:translations_types-list'),
            'last_translation_tag_url': reverse('get_last_translation_tag', args=[0])[:-1],
        })
        super(TranslationAdminForm, self).__init__(*args, **kwargs)

    def clean(self):
        cleaned_data = super(TranslationAdminForm, self).clean()

        for language in self.languages_list:
            if cleaned_data.get("text_"+language) == cleaned_data.get('auxiliary_text_'+language):
                self.add_error('auxiliary_text_' + language,
                               forms.ValidationError(_('DTSE1')))

        return cleaned_data

    def save(self, commit=False):
        translation = super(TranslationAdminForm, self).save(commit=commit)
        translation.save()
        translation.migration_created = False
        translation.save()
        return translation

    class Meta:
        model = Translation
        fields = "__all__"


class TranslationTypeAdminForm(forms.ModelForm):
    class Meta:
        model = TranslationType
        fields = "__all__"


# -*- coding: utf-8 -*-
# Created by Gustavo Del Negro <gustavodelnegro@gmail.com> on 10/1/16.
from django.core.management import BaseCommand
from django.utils.translation import to_locale
from datetime import datetime
from translation_server.models import Translation
from django.core.management import call_command
from django.conf import settings
import os


class Command(BaseCommand):
    help = "This command generates the translation files, based on the contents of 'Translation' model"
    languages_list = None
    locale_path = None
    BASE_DIR = os.path.join(settings.BASE_DIR, 'locale')
    file_map = {}

    def __create_translation_dirs(self):
        if not os.path.exists(self.locale_path):
            os.makedirs(self.locale_path)
        for language in self.languages_list:
            # call_command('makemessages', '-l', language)
            if not os.path.exists(self.locale_path+"/"+to_locale(language)+"/LC_MESSAGES"):
                os.makedirs(self.locale_path+"/"+to_locale(language)+"/LC_MESSAGES")

    def __create_translation_files(self):
        self.__create_translation_dirs()
        for language in self.languages_list:
            self.file_map.update(
                {
                    language: {
                        'dir': self.locale_path + "/" + to_locale(language) + "/LC_MESSAGES/",
                        'file': open(self.locale_path + "/" + to_locale(language) + "/LC_MESSAGES/django.po", "w")
                    }
                }
            )
        header = """
# SOME DESCRIPTIVE TITLE.
# Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
# This file is distributed under the same license as the PACKAGE package.
# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"Report-Msgid-Bugs-To: \\n"
"POT-Creation-Date: 2016-05-19 14:44+0000\\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n"
"Language-Team: LANGUAGE <LL@li.org>\\n"
"Language: \\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=UTF-8\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Plural-Forms: nplurals=2; plural=(n > 1);\\n"
"""
        translations = Translation.objects.filter()
        for language in self.file_map:
            try:
                self.file_map[language]['file'].write(
                    "%s# File generated in: %s \n\n" % (header, datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                )
                for translation in translations:
                    text = getattr(translation, "text_"+language.replace("-", "_"))
                    self.file_map[language]['file'].write(
                        'msgid "%(tag)s"\nmsgstr "%(text)s"\n\n' % {
                            'tag': translation.tag,
                            'text': text.replace('"', '\\"') if text else ""
                        }
                    )
                    if translation.auxiliary_tag:
                        auxiliary_text = getattr(translation, "auxiliary_text_" + language.replace("-", "_"))
                        self.file_map[language]['file'].write(
                            'msgid "%(tag)s"\nmsgstr "%(text)s"\n\n' % {
                                'tag': translation.auxiliary_tag,
                                'text': auxiliary_text.replace('"', '\\"') if auxiliary_text else ""
                            }
                        )
            except Exception as error:
                self.file_map[language]['file'].close()
                raise error
            else:
                self.file_map[language]['file'].close()
        return True

    def add_arguments(self, parser):
        parser.add_argument('languages_list', nargs='+', type=str)
        parser.add_argument(
            '--locales-dir',
            action='store',
            dest='locales_dir',
            default="",
            help="The locales dir to store/write the *.po files",
        )

    def handle(self, *args, **options):
        self.languages_list = options['languages_list'][0].split(',')
        self.locale_path = options['locales_dir'] if len(options['locales_dir']) > 1 else self.BASE_DIR
        if self.__create_translation_files():
            compile_options = {}
            for lang in self.languages_list:
                compile_options.update({'locale': lang})
            call_command('compilemessages', options=compile_options)
